import json
import os
import sys

# sys.path.append('C:\\src\\carsales\\cyclops-cia\\cyclops-cia')
# sys.path.append('C:\\src\\carsales\\cyclops-cia')

from elasticsearch_dsl.connections import connections
from models.CorrectionMessage import CorrectionMessage
from models.MessageStatus import MessageStatus
from services.MessageProcessor import MessageProcessor
from services.image_service import ImageService
from services.logger import Logger


CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(CURRENT_DIR))


class ElasticsearchService:
    """Elasticsearch service"""

    # global elasticsearch connection configuration
    connections.configure(
        default={
            'hosts': os.environ['Cyclops_Cia_Elasticsearch_Dev_Url'],
            'port': os.environ['Cyclops_Cia_Elasticsearch_Dev_Port']
        },
        dev={
            'hosts': os.environ['Cyclops_Cia_Elasticsearch_Dev_Url'],
            'port': os.environ['Cyclops_Cia_Elasticsearch_Dev_Port']
        }
    )

    def __init__(self):
        self.client = connections.get_connection('dev')
        logger = Logger(__name__)
        self.logger = logger.logger

        # create the mappings in elasticsearch
        CorrectionMessage.init()

    def handlePendingMessages(self):
        messageProcessor = MessageProcessor()
        messages = messageProcessor.getMessages(queueName=os.environ['Cyclops_Cia_Correction_Subscriber_Name'])

        for msg in messages:
            try:
                obj = json.loads(msg)
                status = obj['status']
                if status.lower() == MessageStatus.PENDING.name.lower():
                    # self.logger.debug('Handling pending message: ' + msg)
                    message = CorrectionMessage(meta={'id': obj['id']},
                                                imageUrl=obj['imageUrl'],
                                                confScore=obj['confScore'],
                                                source=obj['source'],
                                                status=obj['status'],
                                                category={'was': obj['category']['was'],
                                                          'correction': obj['category']['correction']})
                    message.save()
            except Exception as error:
                self.logger.error('Failed to read pending message', exc_info=True)

    def handleApprovedMessages(self):
        messageProcessor = MessageProcessor()
        messages = messageProcessor.getMessages(queueName=os.environ['Cyclops_Cia_Approved_Correction_Subscriber_Name'])

        for msg in messages:
            try:
                obj = json.loads(msg)
                status = obj['status']
                if status.lower() == MessageStatus.APPROVED.name.lower():
                    # self.logger.debug('Handling approved message: ' + msg)
                    imgSrvc = ImageService(imageUrl=obj['imageUrl'], saveToFilePath=obj['correction'], msgId=obj['id'])
                    imgSrvc.save()
            except Exception as error:
                self.logger.error('Failed to read approved message', exc_info=True)

